@@ -25,7 +25,10 @@ from PyFin.api.Analysis import MA
 from PyFin.api.Analysis import MAX
 from PyFin.api.Analysis import MIN
 from PyFin.api.Analysis import QUANTILE
+from PyFin.api.Analysis import ALLTRUE
+from PyFin.api.Analysis import ANYTRUE
 from PyFin.api.Analysis import VARIANCE
+from PyFin.api.Analysis import STD
 from PyFin.api.Analysis import NPOSITIVE
 from PyFin.api.Analysis import MAPOSITIVE
 from PyFin.api.Analysis import HIST
@@ -53,6 +56,7 @@ from PyFin.api.Analysis import CSRank
 from PyFin.api.Analysis import CSMean
 from PyFin.api.Analysis import CSMeanAdjusted
 from PyFin.api.Analysis import CSQuantile
+from PyFin.api.Analysis import CSZScore
 
 
 __all__ = ["datesList",
@@ -74,7 +78,10 @@ __all__ = ["datesList",
            "MAX",
            "MIN",
            "QUANTILE",
+           "ALLTRUE",
+           "ANYTRUE",
            "VARIANCE",
+           "STD",
            "NPOSITIVE",
            "MAPOSITIVE",
            "HIST",
@@ -100,4 +107,5 @@ __all__ = ["datesList",
            "CSRank",
            "CSMean",
            "CSMeanAdjusted",
-           "CSQuantile"]
+           "CSQuantile",
+           "CSZScore"]
