@@ -8,12 +8,17 @@ Created on 2015-7-16
 import unittest
 import csv
 import os
+import math
 import numpy as np
 from collections import deque
+from PyFin.Math.Accumulators import Latest
 from PyFin.Math.Accumulators import Shift
+from PyFin.Math.Accumulators import Exp
 from PyFin.Math.Accumulators import MovingMax
 from PyFin.Math.Accumulators import MovingMinimum
 from PyFin.Math.Accumulators import MovingQuantile
+from PyFin.Math.Accumulators import MovingAllTrue
+from PyFin.Math.Accumulators import MovingAnyTrue
 from PyFin.Math.Accumulators import MovingAverage
 from PyFin.Math.Accumulators import MovingPositiveAverage
 from PyFin.Math.Accumulators import MovingNegativeAverage
@@ -24,6 +29,7 @@ from PyFin.Math.Accumulators import MovingSum
 from PyFin.Math.Accumulators import MovingCountedPositive
 from PyFin.Math.Accumulators import MovingCountedNegative
 from PyFin.Math.Accumulators import MovingVariance
+from PyFin.Math.Accumulators import MovingStandardDeviation
 from PyFin.Math.Accumulators import MovingNegativeVariance
 from PyFin.Math.Accumulators import MovingHistoricalWindow
 from PyFin.Math.Accumulators import MovingCorrelation
@@ -50,6 +56,20 @@ class TestStatefulAccumulators(unittest.TestCase):
         np.random.seed(0)
         self.sample = np.random.randn(10000)
 
+    def testLatestValueHolderWithStringValue(self):
+        latest = Latest('x')
+        latest.push({'x': 'aaa'})
+
+        self.assertEqual(latest.value, 'aaa')
+
+    def testLatestValueHolderWithOtherValueHolder(self):
+        test1 = Exp(Latest('x'))
+        test1.push({'x': 2.0})
+        self.assertAlmostEqual(test1.value, math.exp(2.0))
+
+        test1.push({'x': np.nan})
+        self.assertTrue(math.isnan(test1.value))
+
     def testShiftValueHolder(self):
         ma = MovingAverage(10, 'close')
 
@@ -287,6 +307,48 @@ class TestStatefulAccumulators(unittest.TestCase):
                                                                  "Quantile expected:   {1:f}\n"
                                                                  "Quantile calculated: {2:f}".format(i, expected, calculated))
 
+    def testMovingAllTrue(self):
+        window = 3
+
+        underlying = Latest('x') > 0.
+
+        mat = MovingAllTrue(window, dependency=underlying)
+
+        total = np.random.randn(2500)
+        con = deque(maxlen=window)
+        for i, value in enumerate(total):
+            value = float(value)
+            con.append(value)
+            mat.push(dict(x=value))
+
+            calculated = mat.result()
+            expected = np.all(np.array(con) > 0.)
+
+            self.assertEqual(calculated, expected, "at index {0}\n"
+                                                   "Quantile expected:   {1}\n"
+                                                   "Quantile calculated: {2}".format(i, expected, calculated))
+
+    def testMovingAnyTrue(self):
+        window = 3
+
+        underlying = Latest('x') > 0.
+
+        mat = MovingAnyTrue(window, dependency=underlying)
+
+        total = np.random.randn(2500)
+        con = deque(maxlen=window)
+        for i, value in enumerate(total):
+            value = float(value)
+            con.append(value)
+            mat.push(dict(x=value))
+
+            calculated = mat.result()
+            expected = np.any(np.array(con) > 0.)
+
+            self.assertEqual(calculated, expected, "at index {0}\n"
+                                                   "Quantile expected:   {1}\n"
+                                                   "Quantile calculated: {2}".format(i, expected, calculated))
+
     def testMovingCountedPositive(self):
         window = 120
         total = 2500
@@ -401,6 +463,64 @@ class TestStatefulAccumulators(unittest.TestCase):
                                                                  "Var calculated: {2:f}".format(i, expected,
                                                                                                 calculated))
 
+    def testMovingStandardDeviation(self):
+        window = 120
+        total = 2500
+
+        # Test moving population variance
+        mv = MovingStandardDeviation(window, dependency='z', isPopulation=True)
+        runningSum = 0.0
+        runningSumSquare = 0.0
+        con = []
+        for i in range(total):
+            value = float(i)
+            con.append(value)
+            mv.push(dict(z=value))
+            runningSum += value
+            runningSumSquare += value * value
+
+            if i >= window:
+                runningSum -= con[0]
+                runningSumSquare -= con[0] * con[0]
+                con = con[1:]
+
+            if i >= window - 1:
+                expected = math.sqrt((runningSumSquare - runningSum * runningSum / window) / window)
+                calculated = mv.result()
+                self.assertAlmostEqual(calculated, expected, 15, "at index {0:d}\n"
+                                                                 "Var expected:   {1:f}\n"
+                                                                 "Var calculated: {2:f}".format(i, expected,
+                                                                                                calculated))
+
+        # Test moving sample variance
+        mv = MovingVariance(window, dependency='z', isPopulation=False)
+        runningSum = 0.0
+        runningSumSquare = 0.0
+        con = []
+        for i in range(total):
+            value = float(i)
+            con.append(value)
+            mv.push(dict(z=value))
+            runningSum += value
+            runningSumSquare += value * value
+
+            if i == 0:
+                self.assertTrue(np.isnan(mv.result()))
+
+            if i >= window:
+                runningSum -= con[0]
+                runningSumSquare -= con[0] * con[0]
+                con = con[1:]
+
+            length = window if i >= window else (i + 1)
+            if i >= window - 1:
+                expected = (runningSumSquare - runningSum * runningSum / length) / (length - 1)
+                calculated = mv.result()
+                self.assertAlmostEqual(calculated, expected, 15, "at index {0:d}\n"
+                                                                 "Var expected:   {1:f}\n"
+                                                                 "Var calculated: {2:f}".format(i, expected,
+                                                                                                calculated))
+
     def testMovingNegativeVariancer(self):
         # test without enough negative value
         mv = MovingNegativeVariance(20, dependency='z', isPopulation=True)
@@ -452,8 +572,8 @@ class TestStatefulAccumulators(unittest.TestCase):
                 with self.assertRaises(ValueError):
                     _ = mh[1]
 
-            for k in range(mh.size):
-                expected = benchmarkContainer[mh.size - 1 - k]
+            for k in range(mh.size()):
+                expected = benchmarkContainer[mh.size() - 1 - k]
                 calculated = mh[k]
                 self.assertAlmostEqual(expected, calculated)
 
@@ -470,8 +590,8 @@ class TestStatefulAccumulators(unittest.TestCase):
                 with self.assertRaises(ValueError):
                     _ = mh[1]
 
-            for k in range(mh.size):
-                expected = benchmarkContainer[mh.size - 1 - k]
+            for k in range(mh.size()):
+                expected = benchmarkContainer[mh.size() - 1 - k]
                 calculated = mh[k]
                 self.assertAlmostEqual(expected, calculated, 12, "at index {0} and position {1}\n"
                                                                  "expected:   {2}\n"
@@ -608,13 +728,11 @@ class TestStatefulAccumulators(unittest.TestCase):
         window = 10
         ms = MovingSkewness(window, dependency='x')
 
-        con = []
+        con = deque(maxlen=window)
         for i, value in enumerate(self.sample):
             con.append(value)
             ms.push(dict(x=value))
-            if i >= window:
-                con = con[1:]
-            if i >= window - 1:
+            if i >= 1:
                 calculated = ms.result()
                 this_moment3 = np.mean(np.power(np.abs(np.array(con) - np.mean(con)), 3))
                 expected = this_moment3 / np.power(np.std(con), 3)
@@ -631,9 +749,8 @@ class TestStatefulAccumulators(unittest.TestCase):
         for i, value in enumerate(self.sample):
             con.append(value)
             maxPos.push(dict(x=value))
-            if i >= window:
-                con = con[1:]
             if i >= window - 1:
+                con = con[-10:]
                 calculated = maxPos.result()
                 expected = con.index(np.max(con))
                 self.assertEqual(expected, calculated, "at index {0:d}\n"
@@ -649,9 +766,8 @@ class TestStatefulAccumulators(unittest.TestCase):
         for i, value in enumerate(self.sample):
             con.append(value)
             minPos.push(dict(x=value))
-            if i >= window:
-                con = con[1:]
             if i >= window - 1:
+                con = con[-10:]
                 calculated = minPos.result()
                 expected = con.index(np.min(con))
                 self.assertEqual(expected, calculated, "at index {0:d}\n"
@@ -663,13 +779,12 @@ class TestStatefulAccumulators(unittest.TestCase):
         window = 10
         mk = MovingKurtosis(window, dependency='x')
 
-        con = []
+        con = deque(maxlen=window)
         for i, value in enumerate(self.sample):
             con.append(value)
             mk.push(dict(x=value))
-            if i >= window:
-                con = con[1:]
-            if i >= window - 1:
+
+            if i >= 1:
                 calculated = mk.result()
                 this_moment4 = np.mean(np.power(np.abs(np.array(con) - np.mean(con)), 4))
                 expected = this_moment4 / np.power(np.std(con), 4)
@@ -697,7 +812,7 @@ class TestStatefulAccumulators(unittest.TestCase):
                                                                                                calculated))
 
     def testMACD(self):
-        macd = MACD(short=5, long=10, dependency='close')
+        macd = MACD(short_win=5, long_win=10, dependency='close')
         short_average = XAverage(window=5, dependency='close')
         long_average = XAverage(window=10, dependency='close')
 
@@ -730,9 +845,9 @@ class TestStatefulAccumulators(unittest.TestCase):
             expected = macd.value - ema_macd.value
             calculated = macd_diff.value
             self.assertAlmostEqual(expected, calculated, 10, "at index {0:d}\n"
-                                                             "expected ema macd diff:   {1:f}\n"
-                                                             "calculated ema macd diff: {2:f}".format(i, expected,
-                                                                                                      calculated))
+                                                            "expected ema macd diff:   {1:f}\n"
+                                                            "calculated ema macd diff: {2:f}".format(i, expected,
+                                                                                                     calculated))
 
     def testMovingRank(self):
         window = 10
@@ -757,13 +872,11 @@ class TestStatefulAccumulators(unittest.TestCase):
         d = 3
         mk = MovingKDJ(window, dependency='x')
 
-        con = []
+        con = deque(maxlen=window)
         this_j = []
         for i, value in enumerate(self.sample):
             con.append(value)
             mk.push(dict(x=value))
-            if i >= window:
-                con = con[1:]
             if len(con) == 1:
                 this_j.append(np.nan)
             else:
